/*!
* @file gpredict.h
*
* Permet la connexion directe Wifi depuis GPredict, sans passer par hamlib/rotctld.
* Pour carte Wemos D1 Mini Pro
*
*/

#ifndef LIBRARIES_GPREDICT_H_
#define LIBRARIES_GPREDICT_H_

#include <ESP8266WiFi.h>
#include <ArduinoWiFiServer.h>
#include <Arduino.h>
#include <WString.h>
#include "rotator_pins.h"
#include "globals.h"

#ifndef STASSID
#define STASSID "GlikJB"
#define STAPSK  "__PASSWORD__"
#define PORT    4533
#endif

const char* ssid     = STASSID;
const char* password = STAPSK;

ArduinoWiFiServer server(PORT);

class gpredict {
public:

    /**************************************************************************/
    /*!
        @brief    Initialization
    */
    /**************************************************************************/
    void gpredict_init() {
        WiFi.begin(ssid, password);
        Serial.print("Connecting to ");
        Serial.print(ssid);
        while (WiFi.status() != WL_CONNECTED) {
            delay(500);
            Serial.print(".");
        }
        IPAddress ip = WiFi.localIP();
        digitalWrite(WIFI_LED, HIGH);       // Wifi LED on

        Serial.println();
        Serial.print("Listening on ");
        Serial.print(WiFi.localIP());
        Serial.print(":");
        Serial.println(PORT);

        server.begin();
    }

    /**************************************************************************/
    /*!
        @brief    Get the commands from telnet and response to the client
    */
    /**************************************************************************/
    void gpredict_proc() {
        char buffer[BUFFER_SIZE];
        char incomingByte;
        char *Data = buffer;
        char *rawData;
        static uint16_t BufferCnt = 0;
        char data[100];
        String str1, str2, str3;

        // Returns first client which has data to read or a 'false' client
        WiFiClient client = server.available();

        // client is true only if it is connected and has data to read
        if (client) {
            ESP.wdtFeed();
            incomingByte = client.read();
            str3 = String("\n");

            // Read new data until '\n'
            if (incomingByte == '\n' || incomingByte == '\r') {
                buffer[BufferCnt] = 0;
                if (buffer[0] == 'p') {
                    // Send current absolute position in deg
                    str1 = String(control_az.input, 1);
                    str2 = String(control_el.input, 1);
                    client.print(str1 + "\n" + str2 + "\n");
                }
                else if (buffer[0] == 'P' && buffer[1] == ' ') {
                    client.print("RPRT 0\n");
                    // Get the absolute position in deg for azimuth
                    rotator.control_mode = position;
                    rawData = strtok_r(Data, " ", &Data);
                    strncpy(data, rawData + 2, 10);
                    char* az = strtok(data, " ");
                    char* el = strtok(NULL, "\0");

                    if (isNumber(az) && isNumber(el)) {
                        control_az.setpoint = atof(az);
                        control_el.setpoint = atof(el);
                    }
                }
                else if (buffer[0] == 'S') {
                    client.print("RPRT 0\n");
                    // Stop Moving
                    rotator.control_mode = position;
                    control_az.setpoint = control_az.input;
                    control_el.setpoint = control_el.input;
                }

                // Reset the buffer an clean the buffer
                BufferCnt = 0;
                //server.flush(); // flush the buffers
            } else {
                // Fill the buffer with incoming data
                buffer[BufferCnt] = incomingByte;
                BufferCnt++;
            }
        }
    }

private:
    bool isNumber(char *input) {
        for (uint16_t i = 0; input[i] != '\0'; i++) {
            if (isalpha(input[i]))
                return false;
        }
        return true;
    }
};

#endif /* LIBRARIES_GPREDICT_H_ */
